@@ -15,7 +15,6 @@ import base64
 import asyncio
 from datetime import datetime
 import logging
-import re
 
 import httpx
 from dracoon.client.models import ProxyConfig
@@ -24,7 +23,7 @@ from dracoon.errors import (MissingCredentialsError, HTTPBadRequestError, HTTPUn
                             HTTPPaymentRequiredError, HTTPForbiddenError, HTTPNotFoundError, HTTPConflictError, HTTPPreconditionsFailedError,
                             HTTPUnknownError)
 
-USER_AGENT = 'dracoon-python-1.5.3'
+USER_AGENT = 'dracoon-python-1.7.0'
 DEFAULT_TIMEOUT_CONFIG = httpx.Timeout(10, connect=20, read=20)
 
 class OAuth2ConnectionType(Enum):
@@ -55,8 +54,8 @@ class DRACOONClient:
 
     }
 
-    def __init__(self, base_url: str, client_id: str = 'dracoon_legacy_scripting', client_secret: str = '', raise_on_err: bool = False,
-                 proxy_config: ProxyConfig = None):
+    def __init__(self, base_url: str, client_id: str = 'dracoon_legacy_scripting', client_secret: str = '', redirect_uri: str = None,
+                 raise_on_err: bool = False, proxy_config: ProxyConfig = None):
         """ client is initialized with DRACOON instance details (url and OAuth client credentials) """
         self.base_url = base_url
         self.client_id = client_id
@@ -65,10 +64,13 @@ class DRACOONClient:
         self.uploader = httpx.AsyncClient(timeout=DEFAULT_TIMEOUT_CONFIG, proxies=proxy_config)
         self.downloader = httpx.AsyncClient(timeout=DEFAULT_TIMEOUT_CONFIG, proxies=proxy_config)
         self.connected = False
+        if redirect_uri:
+            self.redirect_uri = redirect_uri
+        else:
+            self.redirect_uri = f"{self.base_url}/oauth/callback"
         self.connection: DRACOONConnection = None
         self.raise_on_err = raise_on_err
         self.logger = logging.getLogger('dracoon.client')
-
         self.logger.info("DRACOON client created.")
         self.logger.debug(f"DRACOON client config: {self.base_url} // {self.client_id}")
 
@@ -81,7 +83,7 @@ class DRACOONClient:
         except RuntimeError:
             loop = None
         
-        # close httpx client 
+        # close httpx clients
         if loop and loop.is_running():
             loop.create_task(self.http.aclose())
             loop.create_task(self.uploader.aclose())
@@ -142,13 +144,10 @@ class DRACOONClient:
 
         if connection_type == OAuth2ConnectionType.auth_code:
             
-            
             if redirect_uri:
-                _redirect_uri = redirect_uri
-            else:
-                _redirect_uri = self.base_url + '/oauth/callback'
+                self.redirect_uri = redirect_uri
 
-            data = {'grant_type': 'authorization_code', 'code': auth_code, 'client_id': self.client_id, 'client_secret': self.client_secret, 'redirect_uri': _redirect_uri}
+            data = {'grant_type': 'authorization_code', 'code': auth_code, 'client_id': self.client_id, 'client_secret': self.client_secret, 'redirect_uri': self.redirect_uri}
 
             try:
                 res = await self.http.post(url=token_url, data=data)
@@ -197,16 +196,17 @@ class DRACOONClient:
         self.connected = True
         self.http.headers["Authorization"] = "Bearer " + self.connection.access_token
   
-
         return self.connection
 
     async def disconnect(self):
         await self.http.aclose()
+        await self.downloader.aclose()
+        await self.uploader.aclose()
 
     def get_code_url(self):
-        """ builds OAuth authorization code url to visit – requires OAuth app to use redirect uri ($host/oauth/callback) """
+        """ builds OAuth authorization code url to visit – requires OAuth app to use redirect uri ($host/oauth/callback or custom uri) """
         # generate URL string for OAuth auth code flow
-        return self.base_url + f'/oauth/authorize?branding=full&response_type=code&client_id={self.client_id}&redirect_uri={self.base_url}/oauth/callback&scope=all'
+        return self.base_url + f'/oauth/authorize?branding=full&response_type=code&client_id={self.client_id}&redirect_uri={self.redirect_uri}&scope=all'
 
 
     async def logout(self, revoke_refresh_token: bool = False):
@@ -231,11 +231,8 @@ class DRACOONClient:
 
         self.connected = False
         self.connection = None
-        await self.http.aclose()
-        await self.downloader.aclose()
-        await self.uploader.aclose()
+        await self.disconnect()
 
-    
     async def check_access_token(self, test: bool = False):
         """ check access token validity (based on connection time and token validity) """
         self.logger.debug("Testing access token validity.")
@@ -292,7 +289,7 @@ class DRACOONClient:
             self.logger.debug("%s", err.response.content)
 
         if close_client:
-            await self.http.aclose()
+            await self.disconnect()
         
         if raise_on_err:
             self.raise_http_error(err=err)
@@ -300,14 +297,14 @@ class DRACOONClient:
     async def handle_connection_error(self, err: httpx.RequestError):
         self.logger.critical("Connection error.")
         self.logger.critical(err.request.url)
-        await self.http.aclose()
+        await self.disconnect()
         raise err
     
     async def handle_generic_error(self, err: Exception, close_client: bool = False):
         self.logger.critical("An error ocurred.")
         self.logger.debug("%s", err)
         if close_client:
-            await self.http.aclose()
+            await self.disconnect()
         raise err
 
     def raise_http_error(self, err: httpx.HTTPStatusError):
