@@ -5,7 +5,7 @@ Core models required by other models
 
 
 from pydantic import BaseModel
-from typing import Any, Optional, List
+from typing import Dict, Optional, Union
 from datetime import datetime
 
 
@@ -24,3 +24,5 @@ class ErrorMessage(BaseModel):
     debugInfo: Optional[str]
     errorCode: Optional[int]
 
+# adheres to proxy model from httpx (dict with http / https and respective str) or single str
+ProxyConfig = Union[Dict[str, str], str]
\ No newline at end of file
