@@ -36,12 +36,16 @@ class DRACOONUser:
 
     def __init__(self, dracoon_client: DRACOONClient):
         """ requires a DRACOONClient to perform any request """
+         
         if not isinstance(dracoon_client, DRACOONClient):
             raise InvalidClientError(message='Invalid DRACOON client format.')
+        
+        self.logger = logging.getLogger('dracoon.user')
+        
         if dracoon_client.connection:
             self.dracoon = dracoon_client
             self.api_url = self.dracoon.base_url + self.dracoon.api_base_url + '/user'
-            self.logger = logging.getLogger('dracoon.user')
+            
             if self.dracoon.raise_on_err:
                 self.raise_on_err = True
             else:
@@ -63,7 +67,7 @@ class DRACOONUser:
             raise_on_err = True
 
         try:
-            api_url = self.api_url + f'/account/?{str(more_info).lower()}'
+            api_url = self.api_url + f'/account/?more_info={str(more_info).lower()}'
             res = await self.dracoon.http.get(api_url)
             res.raise_for_status()
         except httpx.RequestError as e:
